# Generated by Django 2.2.18 on 2021-02-15 16:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0026_client_multiple_response_types'),
    ]

    operations = [
        migrations.AddField(
            model_name='token',
            name='token_type',
            field=models.CharField(choices=[('DPoP', 'Demonstrating Proof of Posession at the Application Layer Token'), ('bearer', 'Bearer Token')], default='bearer', max_length=50),
        ),
    ]
