# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-05 15:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oidc_provider', '0018_hybridflow_and_clientattrs'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client',
            name='client_secret',
            field=models.CharField(blank=True, max_length=255, verbose_name='Client SECRET'),
        ),
    ]
